@@ -18,6 +18,7 @@ In flux ;-> See C<parse_multipart_form> in L<PApp>.
  fh         filehandle to use (only read()) is ever called
  boundary   the "file"-part boundary
  rsize      max. number of bytes to read
+ bufsize    the approx. buffer size (def. 32768)
 
 =head2 supported methods
 
@@ -29,7 +30,7 @@ In flux ;-> See C<parse_multipart_form> in L<PApp>.
 
 =cut
 
-$VERSION = 0.12;
+$VERSION = 0.121;
 
 no utf8;
 
@@ -39,7 +40,7 @@ sub new {
    my $class = shift;
    my $fh = local *PApp_FormBuffer;
    my $self = tie $fh, $class, {
-      bufsize => 4096,
+      bufsize => 32768,
       @_,
       buffer => "\15\12",
       datalen => 2,
@@ -62,39 +63,49 @@ sub EOF {
 
 sub _datalen {
    my $self = shift;
-   $self->{datalen} = index $self->{buffer}, "--$self->{boundary}";
+   $self->{datalen} = index $self->{buffer}, "\15\12--$self->{boundary}";
    $self->{datalen} = length $self->{buffer} if $self->{datalen} < 0;
 }
 
-sub skip_boundary {
-   my $self = shift;
-   while ($self->{datalen} >= length ($self->{buffer})) {
-      if (length($self->{buffer}) > 4+length($self->{boundary})) {
-         $self->{buffer} = ""; $self->{datalen} = 0;
-      }
-      $self->_refill($self->{bufsize});
-   }
-   $self->{buffer} =~ s/.*?--\Q$self->{boundary}//s;
-   $self->_datalen;
-   #$self->_refill($self->{bufsize});
-   $self->READLINE ne "--";
-}
-
 sub _refill {
    my $self = shift;
    my $len = shift;
 
-   while ($self->{datalen} < $len && $self->{datalen} >= length($self->{buffer})) {
+   while ($self->{datalen} < $len
+	  && $self->{datalen}
+	  && $self->{datalen} >= length($self->{buffer})
+	 ) {
       my $buf;
       $len = $self->{rsize} if $len > $self->{rsize};
-      my $len = $self->{fh}->read($buf, $len);
-      $len > 0 or die "unable to read more form data into FormBuffer: $!\n";
-      $self->{rsize} -= $len;
-      $self->{buffer} .= $buf;
+      my $got = $self->{fh}->read($self->{buffer}, $len, length $self->{buffer});
+
+      $got > 0 or die "unable to read more form data into FormBuffer: $!\n";
+      $self->{rsize} -= $got;
       $self->_datalen;
    }
 }
 
+sub skip_boundary {
+   my $self = shift;
+   while ($self->{datalen} >= length ($self->{buffer})) {
+      if (length($self->{buffer}) > 6 + length($self->{boundary})) {
+         $self->{buffer} = ""; $self->{datalen} = 0;
+      }
+      $self->_refill($self->{bufsize});
+   }
+   $self->{buffer} =~ s/.*?\15\12--\Q$self->{boundary}\E(..)//s;
+   if ($1 eq "--") {
+      return ();
+   } else {
+      if (length $self->{buffer}) {
+	 $self->_datalen;
+      } else {
+	 $self->_refill($self->{bufsize});
+      }
+      return 1;
+   }
+}
+
 sub READ {
    my $self = shift;
    my $buf = \$_[0];
@@ -111,7 +122,7 @@ sub READLINE {
    my $self = shift;
    for(;;) {
       return undef if $self->{datalen} == 0;
-      if ($self->{buffer} =~ s/^([^\15\12]*?)\15?\12//) {
+      if ($self->{buffer} =~ s/^([^\15\12]*?)\15\12//) {
          my $line = $1;
          $self->_datalen;
          return $line;
